@@ -1,14 +1,12 @@
 
-# Note: for principal innovations of this code see fsum.R and fscale.R. Old code is commented out below and was innovated in flag.R.
-#  replaced give.names = TRUE with stub
-
 ckm <- function(x) if(is.double(x)) x else if(is.character(x) && x == "overall.mean") -Inf else stop("mean must be a number or 'overall.mean'") # better than switch !!
 
-fwithin <- function(x, ...) { # g = NULL, w = NULL, na.rm = TRUE, mean = 0,
-  UseMethod("fwithin", x)
-}
+# Note: for principal innovations of this code see fsum.R and fscale.R
+
+fwithin <- function(x, ...) UseMethod("fwithin") # , x
+
 fwithin.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_BW,x,0L,0L,NULL,w,na.rm,ckm(mean),FALSE,FALSE)) else if (is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_BW,x,fnlevels(g),g,NULL,w,na.rm,ckm(mean),FALSE,FALSE)) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -20,12 +18,12 @@ fwithin.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ...)
   }
 }
 fwithin.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, mean = 0, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
+  if(!missing(...)) unused_arg_action(match.call(), ...)
+  g <- if(length(effect) == 1L) .subset2(attr(x, "index"), effect) else finteraction(.subset(attr(x, "index"), effect))
   .Call(Cpp_BW,x,fnlevels(g),g,NULL,w,na.rm,ckm(mean),FALSE,FALSE)
 }
 fwithin.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_BWm,x,0L,0L,NULL,w,na.rm,ckm(mean),FALSE,FALSE)) else if(is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_BWm,x,fnlevels(g),g,NULL,w,na.rm,ckm(mean),FALSE,FALSE)) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -37,7 +35,7 @@ fwithin.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ...) {
   }
 }
 fwithin.data.frame <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_BWl,x,0L,0L,NULL,w,na.rm,ckm(mean),FALSE,FALSE)) else if(is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_BWl,x,fnlevels(g),g,NULL,w,na.rm,ckm(mean),FALSE,FALSE)) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -48,21 +46,23 @@ fwithin.data.frame <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ..
     return(.Call(Cpp_BWl,x,g[[1L]],g[[2L]],g[[3L]],w,na.rm,ckm(mean),FALSE,FALSE))
   }
 }
+fwithin.list <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ...)
+  fwithin.data.frame(x, g, w, na.rm, mean, ...)
 fwithin.pdata.frame <- function(x, effect = 1L, w = NULL, na.rm = TRUE, mean = 0, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
+  if(!missing(...)) unused_arg_action(match.call(), ...)
+  g <- if(length(effect) == 1L) .subset2(attr(x, "index"), effect) else finteraction(.subset(attr(x, "index"), effect))
   .Call(Cpp_BWl,x,fnlevels(g),g,NULL,w,na.rm,ckm(mean),FALSE,FALSE)
 }
 fwithin.grouped_df <- function(x, w = NULL, na.rm = TRUE, mean = 0,
                                keep.group_vars = TRUE, keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x)
-  wsym <- deparse(substitute(w))
+  wsym <- l1orn(as.character(substitute(w)), NULL)
   nam <- attr(x, "names")
   gn2 <- which(nam %in% g[[5L]])
   gn <- if(keep.group_vars) gn2 else NULL
-  if(!(wsym == "NULL" || is.na(wn <- match(wsym, nam)))) {
-    w <- unclass(x)[[wn]]
+  if(!is.null(wsym) && !is.na(wn <- match(wsym, nam))) {
+    w <- .subset2(x, wn)
     if(any(gn2 == wn)) stop("Weights coincide with grouping variables!")
     gn2 <- c(gn2,wn)
     if(keep.w) gn <- c(gn,wn)
@@ -78,31 +78,29 @@ fwithin.grouped_df <- function(x, w = NULL, na.rm = TRUE, mean = 0,
   } else return(.Call(Cpp_BWl,x,g[[1L]],g[[2L]],g[[3L]],w,na.rm,ckm(mean),FALSE,FALSE))
 }
 
+# Within Operator
 
-W <- function(x, ...) { # g = NULL, w = NULL, na.rm = TRUE, mean = 0,
-  UseMethod("W", x)
-}
-W.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ...) {
+W <- function(x, ...) UseMethod("W") # , x
+
+W.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, ...)
   fwithin.default(x, g, w, na.rm, mean, ...)
-}
-W.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, mean = 0, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
-  .Call(Cpp_BW,x,fnlevels(g),g,NULL,w,na.rm,ckm(mean),FALSE,FALSE)
-}
-W.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, stub = "W.", ...) {
+
+W.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, mean = 0, ...)
+  fwithin.pseries(x, effect, w, na.rm, mean, ...)
+
+W.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, stub = "W.", ...)
   add_stub(fwithin.matrix(x, g, w, na.rm, mean, ...), stub)
-}
+
 W.grouped_df <- function(x, w = NULL, na.rm = TRUE, mean = 0,
                          stub = "W.", keep.group_vars = TRUE, keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x)
-  wsym <- deparse(substitute(w))
+  wsym <- l1orn(as.character(substitute(w)), NULL)
   nam <- attr(x, "names")
   gn2 <- which(nam %in% g[[5L]])
   gn <- if(keep.group_vars) gn2 else NULL
-  if(!(wsym == "NULL" || is.na(wn <- match(wsym, nam)))) {
-    w <- unclass(x)[[wn]]
+  if(!is.null(wsym) && !is.na(wn <- match(wsym, nam))) {
+    w <- .subset2(x, wn)
     if(any(gn2 == wn)) stop("Weights coincide with grouping variables!")
     gn2 <- c(gn2,wn)
     if(keep.w) gn <- c(gn, wn)
@@ -120,12 +118,12 @@ W.grouped_df <- function(x, w = NULL, na.rm = TRUE, mean = 0,
 W.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric, na.rm = TRUE, mean = 0,
                           stub = "W.", keep.ids = TRUE, keep.w = TRUE, ...) {
 
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   ax <- attributes(x)
   class(x) <- NULL
   nam <- names(x)
-  g <- if(length(effect) == 1L) unclass(ax[["index"]])[[effect]] else
-    finteraction(unclass(ax[["index"]])[effect])
+  g <- if(length(effect) == 1L) .subset2(ax[["index"]], effect) else
+    finteraction(.subset(ax[["index"]], effect))
 
   if(keep.ids) {
     gn <- which(nam %in% attr(ax[["index"]], "names"))
@@ -135,8 +133,7 @@ W.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric, na.rm = T
   if(!is.null(cols)) cols <- cols2int(cols, x, nam)
 
   if(is.call(w)) {
-    w <- all.vars(w)
-    wn <- ckmatch(w, nam)
+    wn <- ckmatch(all.vars(w), nam)
     w <- x[[wn]]
     cols <- if(is.null(cols)) seq_along(x)[-wn] else cols[cols != wn]
     if(keep.w) gn <- c(gn, wn)
@@ -159,7 +156,7 @@ W.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric, na.rm = T
 W.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE,
                          mean = 0, stub = "W.", keep.by = TRUE, keep.w = TRUE, ...) {
 
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.call(by) || is.call(w)) {
     ax <- attributes(x)
     class(x) <- NULL
@@ -178,13 +175,12 @@ W.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE
     } else {
       gn <- NULL
       if(!is.null(cols)) cols <- cols2int(cols, x, nam)
-      if(!is.GRP(by)) by <- if(is.null(by)) list(0L, 0L, NULL) else if(is.atomic(by)) # Necessary for if by is passed externally !!
+      if(!is.GRP(by)) by <- if(is.null(by)) list(0L, 0L, NULL) else if(is.atomic(by)) # Necessary if by is passed externally !
         at2GRP(by) else GRP.default(by, return.groups = FALSE)
     }
 
     if(is.call(w)) {
-      w <- all.vars(w)
-      wn <- ckmatch(w, nam)
+      wn <- ckmatch(all.vars(w), nam)
       w <- x[[wn]]
       cols <- if(is.null(cols)) seq_along(x)[-wn] else cols[cols != wn]
       if(keep.w) gn <- c(gn, wn)
@@ -197,9 +193,10 @@ W.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE
       ax[["names"]] <- if(is.character(stub)) paste0(stub, nam[cols]) else nam[cols]
       return(setAttributes(.Call(Cpp_BWl,x[cols],by[[1L]],by[[2L]],by[[3L]],w,na.rm,ckm(mean),FALSE,FALSE), ax))
     }
-  } else if(!is.null(cols)) { # Need to do like this, otherwise list-subsetting drops attributes !!
+  } else if(!is.null(cols)) { # Need to do like this, otherwise list-subsetting drops attributes !
     ax <- attributes(x)
-    x <- unclass(x)[cols2int(cols, x, ax[["names"]])]
+    class(x) <- NULL
+    x <- x[cols2int(cols, x, names(x))]
     ax[["names"]] <- names(x)
     setattributes(x, ax)
   }
@@ -216,12 +213,15 @@ W.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE
   }
 }
 
+W.list <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE,
+                         mean = 0, stub = "W.", keep.by = TRUE, keep.w = TRUE, ...)
+  W.data.frame(x, by, w, cols, na.rm, mean, stub, keep.by, keep.w, ...)
+
+
+fbetween <- function(x, ...) UseMethod("fbetween") # , x
 
-fbetween <- function(x, ...) { # g = NULL, w = NULL, na.rm = TRUE, fill = FALSE,
-  UseMethod("fbetween", x)
-}
 fbetween.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_BW,x,0L,0L,NULL,w,na.rm,0,TRUE,fill)) else if (is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_BW,x,fnlevels(g),g,NULL,w,na.rm,0,TRUE,fill)) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -233,12 +233,12 @@ fbetween.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE,
   }
 }
 fbetween.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, fill = FALSE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
+  if(!missing(...)) unused_arg_action(match.call(), ...)
+  g <- if(length(effect) == 1L) .subset2(attr(x, "index"), effect) else finteraction(.subset(attr(x, "index"), effect))
   .Call(Cpp_BW,x,fnlevels(g),g,NULL,w,na.rm,0,TRUE,fill)
 }
 fbetween.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_BWm,x,0L,0L,NULL,w,na.rm,0,TRUE,fill)) else if(is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_BWm,x,fnlevels(g),g,NULL,w,na.rm,0,TRUE,fill)) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -250,7 +250,7 @@ fbetween.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, .
   }
 }
 fbetween.data.frame <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_BWl,x,0L,0L,NULL,w,na.rm,0,TRUE,fill)) else if(is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_BWl,x,fnlevels(g),g,NULL,w,na.rm,0,TRUE,fill)) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -261,21 +261,23 @@ fbetween.data.frame <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALS
     return(.Call(Cpp_BWl,x,g[[1L]],g[[2L]],g[[3L]],w,na.rm,0,TRUE,fill))
   }
 }
+fbetween.list <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, ...)
+  fbetween.data.frame(x, g, w, na.rm, fill, ...)
 fbetween.pdata.frame <- function(x, effect = 1L, w = NULL, na.rm = TRUE, fill = FALSE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
+  if(!missing(...)) unused_arg_action(match.call(), ...)
+  g <- if(length(effect) == 1L) .subset2(attr(x, "index"), effect) else finteraction(.subset(attr(x, "index"), effect))
   .Call(Cpp_BWl,x,fnlevels(g),g,NULL,w,na.rm,0,TRUE,fill)
 }
 fbetween.grouped_df <- function(x, w = NULL, na.rm = TRUE, fill = FALSE,
                                 keep.group_vars = TRUE, keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x)
-  wsym <- deparse(substitute(w))
+  wsym <- l1orn(as.character(substitute(w)), NULL)
   nam <- attr(x, "names")
   gn2 <- which(nam %in% g[[5L]])
   gn <- if(keep.group_vars) gn2 else NULL
-  if(!(wsym == "NULL" || is.na(wn <- match(wsym, nam)))) {
-    w <- unclass(x)[[wn]]
+  if(!is.null(wsym) && !is.na(wn <- match(wsym, nam))) {
+    w <- .subset2(x, wn)
     if(any(gn2 == wn)) stop("Weights coincide with grouping variables!")
     gn2 <- c(gn2,wn)
     if(keep.w) gn <- c(gn,wn)
@@ -292,30 +294,29 @@ fbetween.grouped_df <- function(x, w = NULL, na.rm = TRUE, fill = FALSE,
 }
 
 
-B <- function(x, ...) { # g = NULL, w = NULL, na.rm = TRUE, fill = FALSE,
-  UseMethod("B", x)
-}
-B.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, ...) {
+# Between Operator
+
+B <- function(x, ...) UseMethod("B") # , x
+
+B.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, ...)
   fbetween.default(x, g, w, na.rm, fill, ...)
-}
-B.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, fill = FALSE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
-  .Call(Cpp_BW,x,fnlevels(g),g,NULL,w,na.rm,0,TRUE,fill)
-}
-B.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, stub = "B.", ...) {
+
+B.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, fill = FALSE, ...)
+  fbetween.pseries(x, effect, w, na.rm, fill, ...)
+
+B.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, fill = FALSE, stub = "B.", ...)
   add_stub(fbetween.matrix(x, g, w, na.rm, fill, ...), stub)
-}
+
 B.grouped_df <- function(x, w = NULL, na.rm = TRUE, fill = FALSE,
                          stub = "B.", keep.group_vars = TRUE, keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x)
-  wsym <- deparse(substitute(w))
+  wsym <- l1orn(as.character(substitute(w)), NULL)
   nam <- attr(x, "names")
   gn2 <- which(nam %in% g[[5L]])
   gn <- if(keep.group_vars) gn2 else NULL
-  if(!(wsym == "NULL" || is.na(wn <- match(wsym, nam)))) {
-    w <- unclass(x)[[wn]]
+  if(!is.null(wsym) && !is.na(wn <- match(wsym, nam))) {
+    w <- .subset2(x, wn)
     if(any(gn2 == wn)) stop("Weights coincide with grouping variables!")
     gn2 <- c(gn2,wn)
     if(keep.w) gn <- c(gn, wn)
@@ -330,14 +331,15 @@ B.grouped_df <- function(x, w = NULL, na.rm = TRUE, fill = FALSE,
       }
   } else return(add_stub(.Call(Cpp_BWl,x,g[[1L]],g[[2L]],g[[3L]],w,na.rm,0,TRUE,fill), stub))
 }
+
 B.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric, na.rm = TRUE, fill = FALSE,
                           stub = "B.", keep.ids = TRUE, keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   ax <- attributes(x)
   class(x) <- NULL
   nam <- names(x)
-  g <- if(length(effect) == 1L) unclass(ax[["index"]])[[effect]] else
-    finteraction(unclass(ax[["index"]])[effect])
+  g <- if(length(effect) == 1L) .subset2(ax[["index"]], effect) else
+    finteraction(.subset(ax[["index"]], effect))
 
   if(keep.ids) {
     gn <- which(nam %in% attr(ax[["index"]], "names"))
@@ -347,8 +349,7 @@ B.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric, na.rm = T
   if(!is.null(cols)) cols <- cols2int(cols, x, nam)
 
   if(is.call(w)) {
-    w <- all.vars(w)
-    wn <- ckmatch(w, nam)
+    wn <- ckmatch(all.vars(w), nam)
     w <- x[[wn]]
     cols <- if(is.null(cols)) seq_along(x)[-wn] else cols[cols != wn]
     if(keep.w) gn <- c(gn, wn)
@@ -368,9 +369,10 @@ B.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric, na.rm = T
       return(.Call(Cpp_BWl,`oldClass<-`(x, ax[["class"]]),fnlevels(g),g,NULL,w,na.rm,0,TRUE,fill))
   }
 }
+
 B.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE,
                          fill = FALSE, stub = "B.", keep.by = TRUE, keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.call(by) || is.call(w)) {
     ax <- attributes(x)
     class(x) <- NULL
@@ -389,13 +391,12 @@ B.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE
     } else {
       gn <- NULL
       if(!is.null(cols)) cols <- cols2int(cols, x, nam)
-      if(!is.GRP(by)) by <- if(is.null(by)) list(0L, 0L, NULL) else if(is.atomic(by)) # Necessary if by is passed externally !!
+      if(!is.GRP(by)) by <- if(is.null(by)) list(0L, 0L, NULL) else if(is.atomic(by)) # Necessary if by is passed externally !
         at2GRP(by) else GRP.default(by, return.groups = FALSE)
     }
 
     if(is.call(w)) {
-      w <- all.vars(w)
-      wn <- ckmatch(w, nam)
+      wn <- ckmatch(all.vars(w), nam)
       w <- x[[wn]]
       cols <- if(is.null(cols)) seq_along(x)[-wn] else cols[cols != wn]
       if(keep.w) gn <- c(gn, wn)
@@ -408,9 +409,10 @@ B.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE
       ax[["names"]] <- if(is.character(stub)) paste0(stub, nam[cols]) else nam[cols]
       return(setAttributes(.Call(Cpp_BWl,x[cols],by[[1L]],by[[2L]],by[[3L]],w,na.rm,0,TRUE,fill), ax))
     }
-  } else if(!is.null(cols)) { # Necessary, else attributes are dropped by list-subsetting !!
+  } else if(!is.null(cols)) { # Necessary, else attributes are dropped by list-subsetting !
     ax <- attributes(x)
-    x <- unclass(x)[cols2int(cols, x, ax[["names"]])]
+    class(x) <- NULL
+    x <- x[cols2int(cols, x, names(x))]
     ax[["names"]] <- names(x)
     setattributes(x, ax)
   }
@@ -426,3 +428,7 @@ B.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE
     return(.Call(Cpp_BWl,x,by[[1L]],by[[2L]],by[[3L]],w,na.rm,0,TRUE,fill))
   }
 }
+
+B.list <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE,
+                         fill = FALSE, stub = "B.", keep.by = TRUE, keep.w = TRUE, ...)
+  B.data.frame(x, by, w, cols, na.rm, fill, stub, keep.by, keep.w, ...)
