@@ -1,15 +1,14 @@
-# Make faster ???
+
+# Make faster ?
 cm <- function(x) if(is.double(x)) x else if(is.character(x) && x == "overall.mean") -Inf else if(isFALSE(x)) Inf else stop("mean must be a number, 'overall.mean' or FALSE")
 csd <- function(x) if(is.double(x)) x else if(is.character(x) && x == "within.sd") -Inf else stop("sd must be a number or 'within.sd'")
 
-# w.type = "frequency"
-# Todo: center and scale arguments -> link to fmean and fsd with TRA ??
+# TODO: w.type - Implement reliability weights?
+
+fscale <- function(x, ...) UseMethod("fscale") # , x
 
-fscale <- function(x, ...) { # g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1,
-  UseMethod("fscale", x)
-}
 fscale.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_fscale,x,0L,0L,w,na.rm,cm(mean),csd(sd))) else if (is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_fscale,x,fnlevels(g),g,w,na.rm,cm(mean),csd(sd))) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -21,12 +20,12 @@ fscale.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1
   }
 }
 fscale.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
+  if(!missing(...)) unused_arg_action(match.call(), ...)
+  g <- if(length(effect) == 1L) .subset2(attr(x, "index"), effect) else finteraction(.subset(attr(x, "index"), effect))
   .Call(Cpp_fscale,x,fnlevels(g),g,w,na.rm,cm(mean),csd(sd))
 }
 fscale.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_fscalem,x,0L,0L,w,na.rm,cm(mean),csd(sd))) else if (is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_fscalem,x,fnlevels(g),g,w,na.rm,cm(mean),csd(sd))) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -38,14 +37,14 @@ fscale.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1,
   }
 }
 fscale.grouped_df <- function(x, w = NULL, na.rm = TRUE, mean = 0, sd = 1, keep.group_vars = TRUE, keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x)
-  wsym <- deparse(substitute(w)) # faster way ???
+  wsym <- l1orn(as.character(substitute(w)), NULL)
   nam <- attr(x, "names")
   gn2 <- which(nam %in% g[[5L]])
   gn <- if(keep.group_vars) gn2 else NULL
-  if(!(wsym == "NULL" || is.na(wn <- match(wsym, nam)))) {
-    w <- unclass(x)[[wn]]
+  if(!is.null(wsym) && !is.na(wn <- match(wsym, nam))) {
+    w <- .subset2(x, wn)
     if(any(gn2 == wn)) stop("Weights coincide with grouping variables!")
     gn2 <- c(gn2, wn)
     if(keep.w) gn <- c(gn, wn)
@@ -61,7 +60,7 @@ fscale.grouped_df <- function(x, w = NULL, na.rm = TRUE, mean = 0, sd = 1, keep.
   } else return(.Call(Cpp_fscalel,x,g[[1L]],g[[2L]],w,na.rm,cm(mean),csd(sd)))
 }
 fscale.data.frame <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(g)) return(.Call(Cpp_fscalel,x,0L,0L,w,na.rm,cm(mean),csd(sd))) else if(is.atomic(g)) {
     if(is.nmfactor(g)) return(.Call(Cpp_fscalel,x,fnlevels(g),g,w,na.rm,cm(mean),csd(sd))) else {
       g <- qG(g, ordered = FALSE, na.exclude = FALSE)
@@ -72,39 +71,37 @@ fscale.data.frame <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd
     return(.Call(Cpp_fscalel,x,g[[1L]],g[[2L]],w,na.rm,cm(mean),csd(sd)))
   }
 }
+fscale.list <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...)
+  fscale.data.frame(x, g, w, na.rm, mean, sd, ...)
 fscale.pdata.frame <- function(x, effect = 1L, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
+  if(!missing(...)) unused_arg_action(match.call(), ...)
+  g <- if(length(effect) == 1L) .subset2(attr(x, "index"), effect) else finteraction(.subset(attr(x, "index"), effect))
   .Call(Cpp_fscale,x,fnlevels(g),g,w,na.rm,cm(mean),csd(sd))
 }
 
 
-# Todo: Could still implement return = c("cols","all","add")
+# Standardization Operator
+
+STD <- function(x, ...) UseMethod("STD") # , x
+
+STD.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...)
+  fscale.default(x, g, w, na.rm, mean, sd, ...)
+
+STD.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...)
+  fscale.pseries(x, effect, w, na.rm, mean, sd, ...)
+
+STD.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1, stub = "STD.", ...)
+  add_stub(fscale.matrix(x, g, w, na.rm, mean, sd, ...), stub)
 
-STD <- function(x, ...) { # g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1,
-  UseMethod("STD", x)
-}
-STD.default <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...) {
-  fscale.default(x, g, w, na.rm, mean, sd)
-}
-STD.pseries <- function(x, effect = 1L, w = NULL, na.rm = TRUE, mean = 0, sd = 1, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  g <- if(length(effect) == 1L) unclass(attr(x, "index"))[[effect]] else finteraction(unclass(attr(x, "index"))[effect])
-  .Call(Cpp_fscale,x,fnlevels(g),g,w,na.rm,cm(mean),csd(sd))
-}
-STD.matrix <- function(x, g = NULL, w = NULL, na.rm = TRUE, mean = 0, sd = 1, stub = "STD.", ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  add_stub(fscale.matrix(x, g, w, na.rm, mean, sd), stub)
-}
 STD.grouped_df <- function(x, w = NULL, na.rm = TRUE, mean = 0, sd = 1, stub = "STD.", keep.group_vars = TRUE, keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x)
-  wsym <- deparse(substitute(w))
+  wsym <- l1orn(as.character(substitute(w)), NULL)
   nam <- attr(x, "names")
   gn2 <- which(nam %in% g[[5L]])
   gn <- if(keep.group_vars) gn2 else NULL
-  if(!(wsym == "NULL" || is.na(wn <- match(wsym, nam)))) {
-    w <- unclass(x)[[wn]]
+  if(!is.null(wsym) && !is.na(wn <- match(wsym, nam))) {
+    w <- .subset2(x, wn)
     if(any(gn2 == wn)) stop("Weights coincide with grouping variables!")
     gn2 <- c(gn2,wn)
     if(keep.w) gn <- c(gn,wn)
@@ -120,16 +117,16 @@ STD.grouped_df <- function(x, w = NULL, na.rm = TRUE, mean = 0, sd = 1, stub = "
   } else return(add_stub(.Call(Cpp_fscalel,x,g[[1L]],g[[2L]],w,na.rm,cm(mean),csd(sd)), stub))
 }
 
-# updated (best) version !!
+# updated (best) version !
 STD.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric,
                             na.rm = TRUE, mean = 0, sd = 1, stub = "STD.", keep.ids = TRUE,
                             keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
+  if(!missing(...)) unused_arg_action(match.call(), ...)
   ax <- attributes(x)
   class(x) <- NULL
   nam <- names(x)
-  g <- if(length(effect) == 1L) unclass(ax[["index"]])[[effect]] else
-    finteraction(unclass(ax[["index"]])[effect])
+  g <- if(length(effect) == 1L) .subset2(ax[["index"]], effect) else
+    finteraction(.subset(ax[["index"]], effect))
 
   if(keep.ids) {
     gn <- which(nam %in% attr(ax[["index"]], "names"))
@@ -139,8 +136,7 @@ STD.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric,
   if(!is.null(cols)) cols <- cols2int(cols, x, nam)
 
   if(is.call(w)) {
-    w <- all.vars(w)
-    wn <- ckmatch(w, nam, "Unknown weight variable:")
+    wn <- ckmatch(all.vars(w), nam, "Unknown weight variable:")
     w <- x[[wn]]
     cols <- if(is.null(cols)) seq_along(x)[-wn] else cols[cols != wn]
     if(keep.w) gn <- c(gn, wn)
@@ -160,12 +156,13 @@ STD.pdata.frame <- function(x, effect = 1L, w = NULL, cols = is.numeric,
       return(.Call(Cpp_fscalel,`oldClass<-`(x, ax[["class"]]),fnlevels(g),g,w,na.rm,cm(mean),csd(sd)))
   }
 }
-# updated, fast and data.table proof version !!!
+
+# updated, fast and data.table proof version !
 STD.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric,
                            na.rm = TRUE, mean = 0, sd = 1, stub = "STD.", keep.by = TRUE,
                            keep.w = TRUE, ...) {
-  if(!missing(...)) stop("Unknown argument ", dotstostr(...))
-  # fastest solution??
+  if(!missing(...)) unused_arg_action(match.call(), ...)
+
   if(is.call(by) || is.call(w)) {
     ax <- attributes(x)
     class(x) <- NULL
@@ -184,13 +181,12 @@ STD.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric,
     } else {
       gn <- NULL
       if(!is.null(cols)) cols <- cols2int(cols, x, nam)
-      if(!is.GRP(by)) by <- if(is.null(by)) list(0L, 0L, NULL) else if(is.atomic(by)) # Necessary for if by is passed externally !!
+      if(!is.GRP(by)) by <- if(is.null(by)) list(0L, 0L, NULL) else if(is.atomic(by)) # Necessary for if by is passed externally !
         at2GRP(by) else GRP.default(by, return.groups = FALSE)
     }
 
     if(is.call(w)) {
-      w <- all.vars(w)
-      wn <- ckmatch(w, nam, "Unknown weight variable:")
+      wn <- ckmatch(all.vars(w), nam, "Unknown weight variable:")
       w <- x[[wn]]
       cols <- if(is.null(cols)) seq_along(x)[-wn] else cols[cols != wn]
       if(keep.w) gn <- c(gn, wn)
@@ -205,7 +201,8 @@ STD.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric,
     }
   } else if(!is.null(cols)) { # Needs to be like this, otherwise subsetting dropps the attributes !!
     ax <- attributes(x)
-    x <- unclass(x)[cols2int(cols, x, ax[["names"]])]
+    class(x) <- NULL
+    x <- x[cols2int(cols, x, names(x))]
     ax[["names"]] <- names(x)
     setattributes(x, ax)
   }
@@ -221,3 +218,7 @@ STD.data.frame <- function(x, by = NULL, w = NULL, cols = is.numeric,
     return(.Call(Cpp_fscalel,x,by[[1L]],by[[2L]],w,na.rm,cm(mean),csd(sd)))
   }
 }
+
+STD.list <- function(x, by = NULL, w = NULL, cols = is.numeric, na.rm = TRUE, mean = 0, sd = 1, stub = "STD.", keep.by = TRUE, keep.w = TRUE, ...)
+  STD.data.frame(x, by, w, cols, na.rm, mean, sd, stub, keep.by, keep.w, ...)
+
